from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import japr.util
from git import InvalidGitRepositoryError
from git.repo import Repo
import os


class RustCheckProvider(CheckProvider):
    def name(self):
        return "Rust"

    def test(self, directory):
        cargo_tomls = list(japr.util.find_files_with_name(directory, "Cargo.toml"))

        try:
            repo = Repo(directory, search_parent_directories=True)
        except InvalidGitRepositoryError:
            repo = None  # Deal with later when we know what checks we're doing

        if len(cargo_tomls) != 0:
            for cargo_toml in cargo_tomls:
                # Check lock file is committed into Git
                if repo is not None:
                    lock_file = os.path.join(os.path.split(cargo_toml)[0], "Cargo.lock")
                    is_file_committed = any(
                        f.type == "blob"
                        and os.path.join(repo.working_dir, f.path) == lock_file
                        for f in repo.tree("HEAD").list_traverse()
                    )
                    yield CheckResult(
                        "RS004",
                        Result.PASSED if is_file_committed else Result.FAILED,
                        cargo_toml,
                    )
                else:
                    yield CheckResult(
                        "RS004",
                        Result.PRE_REQUISITE_CHECK_FAILED,
                        cargo_toml,
                    )
        else:
            yield CheckResult("RS004", Result.NOT_APPLICABLE)

    def checks(self):
        return [
            Check(
                "RS004",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Rust projects should have their Cargo lock files committed into Git",
                """When using Cargo for Rust, the lock files should be comitted into Git. This ensures that all dependencies of packages are installed at the same version no matter when and on what machine the project is installed.""",
            ),
        ]



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import os


class LicenseCheckProvider(CheckProvider):
    def name(self):
        return "License"

    def test(self, directory):
        yield CheckResult(
            "LI001",
            (
                Result.PASSED
                if any(
                    [
                        os.path.isfile(os.path.join(directory, "LICENSE.md")),
                        os.path.isfile(os.path.join(directory, "LICENSE")),
                        os.path.isfile(os.path.join(directory, "LICENSE.txt")),
                        os.path.isfile(os.path.join(directory, "LICENSE.rst")),
                    ]
                )
                else Result.FAILED
            ),
        )

    def checks(self):
        return [
            Check(
                "LI001",
                Severity.MEDIUM,
                ["open-source", "personal"],
                (
                    "Projects should have a LICENSE.md file describing how the project"
                    " can be used"
                ),
                """Create a LICENSE.md file in the root of the project and add content to describe to other users how this project can be used

If you are not familiar with the different licenses available to you, try https://choosealicense.com which guides you through the choice.""",
            )
        ]



from git import InvalidGitRepositoryError, Repo
from git.util import expand_path
from jinja2 import Environment, PackageLoader
import os
import re


def _convert_ssh_to_http(url):
    if "@" in url:
        return re.sub(r"^.+@(.+):(.+?)(\.git)?$", r"https://\1/\2", url)
    return url


def template(template_file, directory):
    env = Environment(
        loader=PackageLoader("japr"),
    )
    template = env.get_template(template_file)

    try:
        github_remote_url = Repo(directory).remote("origin").url

        github_remote_url = _convert_ssh_to_http(github_remote_url)

        if not github_remote_url.endswith("/"):
            github_remote_url[:-1]
    except InvalidGitRepositoryError:
        github_remote_url = None

    vars = {
        "project_name": os.path.split(directory)[1]
        .title()
        .replace("-", " ")
        .replace("_", " "),
        "github_remote_url": github_remote_url,
    }

    return template.render(vars)



from japr.check_providers.readme_check_provider import ReadmeCheckProvider
from japr.check_providers.license_check_provider import LicenseCheckProvider
from japr.check_providers.git_check_provider import GitCheckProvider
from japr.check_providers.ci_check_provider import CiCheckProvider
from japr.check_providers.python_check_provider import PythonCheckProvider
from japr.check_providers.github_check_provider import GitHubCheckProvider
from japr.check_providers.csharp_check_provider import CSharpCheckProvider
from japr.check_providers.contributing_check_provider import ContributingCheckProvider
from japr.check_providers.javascript_check_provider import JavascriptCheckProvider
from japr.check_providers.rust_check_provider import RustCheckProvider
from japr.check_providers.terraform_check_provider import TerraformCheckProvider
from japr.check_providers.shell_check_provider import ShellCheckProvider

check_providers = [
    ReadmeCheckProvider(),
    LicenseCheckProvider(),
    GitCheckProvider(),
    CiCheckProvider(),
    PythonCheckProvider(),
    GitHubCheckProvider(),
    CSharpCheckProvider(),
    ContributingCheckProvider(),
    JavascriptCheckProvider(),
    RustCheckProvider(),
    TerraformCheckProvider(),
    ShellCheckProvider(),
]



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import japr.util
from git import InvalidGitRepositoryError
from git.repo import Repo
import os
import stat

IDE_DIRECTORIES = [".vs", ".idea", ".settings"]  # Visual Studio  # Intellij  # Eclipse

IDE_FILES = [".classpath", ".project"]  # Eclipse

IDE_FILE_EXTENSIONS = ["swp", "iml", "iws", "ipr"]  # Vim  # Intellij


class GitCheckProvider(CheckProvider):
    def name(self):
        return "Git"

    def test(self, directory):
        try:
            repo = Repo(directory, search_parent_directories=True)
            yield CheckResult("GI001", Result.PASSED)

            yield CheckResult(
                "GI002", Result.PASSED if "origin" in repo.remotes else Result.FAILED
            )
            yield CheckResult(
                "GI003", Result.PASSED if "master" not in repo.heads else Result.FAILED
            )
            yield CheckResult(
                "GI004",
                (
                    Result.PASSED
                    if os.path.isfile(os.path.join(directory, ".gitignore"))
                    else Result.FAILED
                ),
            )

            ds_store_paths = [
                f.path
                for f in repo.tree("HEAD").list_traverse()
                if f.type == "blob" and f.name == ".DS_Store"
            ]
            if len(ds_store_paths) != 0:
                for ds_store_path in ds_store_paths:
                    yield CheckResult("GI005", Result.FAILED, ds_store_path)
            else:
                yield CheckResult("GI005", Result.PASSED)

            ide_paths = [
                f.path
                for f in repo.tree("HEAD").list_traverse()
                if (f.type == "tree" and f.name in IDE_DIRECTORIES)
                or (f.type == "blob" and f.name in IDE_FILES)
                or (
                    f.type == "blob"
                    and os.path.splitext(f.name)[1] in IDE_FILE_EXTENSIONS
                )
            ]
            if len(ide_paths) != 0:
                for ide_path in ide_paths:
                    yield CheckResult("GI006", Result.FAILED, ide_path)
            else:
                yield CheckResult("GI006", Result.PASSED)

            shell_files = set(japr.util.find_executable_files(directory))
            shell_files.update(japr.util.find_files_with_shebang(directory))

            for shell_file in shell_files:
                with open(shell_file, "r") as f:
                    first_line = f.readline().strip()
                    if first_line.startswith("#!") and not "x" in stat.filemode(
                        (
                            repo.tree("HEAD")
                            / os.path.relpath(shell_file, repo.working_dir)
                        ).mode
                    ):
                        yield CheckResult(
                            "GI007",
                            Result.FAILED,
                            shell_file,
                        )
                    else:
                        yield CheckResult("GI007", Result.PASSED, shell_file)
            else:
                yield CheckResult("GI007", Result.NOT_APPLICABLE)

        except InvalidGitRepositoryError:
            yield CheckResult("GI001", Result.FAILED)
            yield CheckResult("GI002", Result.PRE_REQUISITE_CHECK_FAILED)
            yield CheckResult("GI003", Result.PRE_REQUISITE_CHECK_FAILED)
            yield CheckResult("GI004", Result.PRE_REQUISITE_CHECK_FAILED)
            yield CheckResult("GI005", Result.PRE_REQUISITE_CHECK_FAILED)
            yield CheckResult("GI006", Result.PRE_REQUISITE_CHECK_FAILED)
            yield CheckResult("GI007", Result.PRE_REQUISITE_CHECK_FAILED)

    def checks(self):
        return [
            Check(
                "GI001",
                Severity.HIGH,
                ["open-source", "inner-source", "team", "personal"],
                "Projects should be tracked in Git version control",
                """All projects, even the smallest personal projects benefit from being tracked in Git as it provides branch management, backups and history to your project.

Run `git init` in this project to setup Git and then make a commit.""",
            ),
            Check(
                "GI002",
                Severity.HIGH,
                ["open-source", "inner-source", "team", "personal"],
                "Projects in Git should have a remote copy in origin",
                """This project does not have a Git remote named 'origin' which suggests there is no backup copy of the project should it be lost.

Setup a Git repository on your favourite Git service (e.g. GitHub) and follow the instructions to add a remote to an existing project. The instructions will likely look like:

git remote add origin <your repository url>
git push origin main""",
            ),
            Check(
                "GI003",
                Severity.HIGH,
                ["open-source", "inner-source", "team", "personal"],
                (
                    "Projects in Git should switch from a 'master' branch to a 'main'"
                    " branch"
                ),
                """This project has a branch named 'master' however it is now recommended to use a branch named 'main' to avoid culturally inappropriate language.

The following guide does a good job or describing the process and providing solutions to any issues you may have with this: https://www.git-tower.com/learn/git/faq/git-rename-master-to-main""",
            ),
            Check(
                "GI004",
                Severity.LOW,
                ["open-source", "inner-source", "team", "personal"],
                "Projects in Git should have a .gitignore file",
                """.gitignore files help you avoid committing unwanted files into Git such as binaries or build artifacts. You should create a .gitignore file for this project.

You can find comprehensive examples for your chosen language here: https://github.com/github/gitignore""",
            ),
            Check(
                "GI005",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Avoid committing .DS_store files",
                """.DS_store files are OSX metadata files in a proprietary binary format. When committed to Git repositories they cause unnecessary changes and provide no value as they differ per machine.

You can tell git to ignore them from commits by adding them to your .gitignore.

You can also all them to your global .gitignore to avoid ever committing them in any repository. Configure a global .gitignore using the following:
git config --global core.excludesfile ~/.gitignore

To remove one from the current repository you can use:
git rm --cached ./path/to/.DS_Store""",
            ),
            Check(
                "GI006",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Avoid committing IDE related files/directories",
                """Many IDEs store IDE specific files with your project. When committed to Git repositories they cause unnecessary changes and provide no value as they differ per machine.

You can tell git to ignore them from commits by adding them to your .gitignore.

You can also all them to your global .gitignore to avoid ever committing them in any repository. Configure a global .gitignore using the following:
git config --global core.excludesfile ~/.gitignore

To remove one from the current repository you can use:
git rm --cached /path/to/file""",
            ),
            Check(
                "GI007",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Executable files should be stored in Git with the executable flag set",
                """Linux and OSX systems use the executable flag to determine if a file is executable. Usually git maintains a similar flag with each file however Windows does not have a concept of this and so executable files can be committed into git without the correct executable flag, making them not executable on Linux and OSX. If you have a shell script or other executable file, ensure it is stored in Git with the executable flag set.

                To mark the file as executable in git, use: git update-index --chmod=+x ./path/to/file""",
            ),
        ]



from japr.check import Check, CheckProvider, CheckFix, CheckResult, Result, Severity
import japr.template_util
import os


class AddReadmeFix(CheckFix):
    def fix(self, directory, _):
        with open(os.path.join(directory, "README.md"), "w") as f:
            f.write(japr.template_util.template("README.md", directory))
        return True

    @property
    def success_message(self):
        return (
            "Created a README.md file in the root directory from a template. You should"
            " add your own content to it."
        )

    @property
    def failure_message(self):
        return (
            "Tried to create a README.md file in the root directory but was unable to."
        )


class ReadmeCheckProvider(CheckProvider):
    def name(self):
        return "Readme"

    def test(self, directory):
        if os.path.isfile(os.path.join(directory, "README.md")):
            readme_path = os.path.join(directory, "README.md")
        elif os.path.isfile(os.path.join(directory, "README")):
            readme_path = os.path.join(directory, "README")
        elif os.path.isfile(os.path.join(directory, "README.txt")):
            readme_path = os.path.join(directory, "README.txt")
        elif os.path.isfile(os.path.join(directory, "README.rst")):
            readme_path = os.path.join(directory, "README.rst")
        else:
            readme_path = None

        yield CheckResult(
            "RE001",
            Result.PASSED if readme_path is not None else Result.FAILED,
            fix=AddReadmeFix(),
        )

        if readme_path is not None:
            with open(readme_path, "r") as readme_file:
                content = readme_file.read()

            if readme_path.endswith(".rst"):
                yield CheckResult(
                    "RE002",
                    (
                        Result.PASSED
                        if content.find("Install\n=") != -1
                        or content.find("Setup\n=") != -1
                        or content.find("Getting Started\n=") != -1
                        or content.find("Quickstart\n=") != -1
                        else Result.FAILED
                    ),
                )
                yield CheckResult(
                    "RE003",
                    (
                        Result.PASSED
                        if content.find("Usage\n=") != -1
                        or content.find("How-to\n=") != -1
                        or content.find("API\n=") != -1
                        else Result.FAILED
                    ),
                )
            else:
                yield CheckResult(
                    "RE002",
                    (
                        Result.PASSED
                        if content.find("# Install") != -1
                        or content.find("# Setup") != -1
                        or content.find("# Getting Started") != -1
                        or content.find("# Quickstart") != -1
                        else Result.FAILED
                    ),
                )
                yield CheckResult(
                    "RE003",
                    (
                        Result.PASSED
                        if content.find("# Usage") != -1
                        or content.find("# How-to") != -1
                        or content.find("# API") != -1
                        else Result.FAILED
                    ),
                )
        else:
            yield CheckResult("RE002", Result.PRE_REQUISITE_CHECK_FAILED)
            yield CheckResult("RE003", Result.PRE_REQUISITE_CHECK_FAILED)

    def checks(self):
        return [
            Check(
                "RE001",
                Severity.HIGH,
                ["open-source", "inner-source", "team", "personal"],
                (
                    "Projects should have a README.md file describing the project and"
                    " its use"
                ),
                """Create a README.md file in the root of the project and add content to describe to other users (or just your future self) things like:
- Why does this project exist?
- How do I install it?
- How do I use it?
- What configuration can be set?
- How do I build the source code?

See https://www.makeareadme.com/ for further guidance""",
            ),
            Check(
                "RE002",
                Severity.LOW,
                ["open-source", "inner-source", "team", "personal"],
                "README.md should contain an Installation section",
                """To help users (and your future self) install your project/library you should provide an installation section in your README. Add the following to your readme:

## Installation
1. Do this
2. Now do this""",
            ),
            Check(
                "RE003",
                Severity.LOW,
                ["open-source", "inner-source", "team", "personal"],
                "README.md should contain a Usage section",
                """To help users (and your future self) use your project/library you should provide a usage section in your README. Add the following to your readme:

## Usage
To do this thing:
1. Do this
2. Then run this""",
            ),
        ]



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import japr.util
from git import InvalidGitRepositoryError
from git.repo import Repo
import os
import json


_linters = [
    "jslint",
    "eslint",
    "jshint",
    "jscs",
    "standard",
]


def _extract_dependencies_from_package_json(file):
    with open(file) as f:
        data = json.load(f)

        try:
            dependencies = [dependency for dependency in data["dependencies"]]
        except KeyError:
            dependencies = []
        try:
            dev_dependencies = [
                dev_dependency for dev_dependency in data["devDependencies"]
            ]
        except KeyError:
            dev_dependencies = []

        return set(dependencies).union(set(dev_dependencies))


class JavascriptCheckProvider(CheckProvider):
    def name(self):
        return "Javascript"

    def test(self, directory):
        package_jsons = list(japr.util.find_files_with_name(directory, "package.json"))

        try:
            repo = Repo(directory, search_parent_directories=True)
        except InvalidGitRepositoryError:
            repo = None  # Deal with later when we know what checks we're doing

        if len(package_jsons) != 0:
            for package_json in package_jsons:
                dependencies = _extract_dependencies_from_package_json(
                    os.path.join(directory, package_json)
                )
                yield CheckResult(
                    "JS002",
                    (
                        Result.PASSED
                        if len(set(_linters).intersection(dependencies))
                        else Result.FAILED
                    ),
                    package_json,
                )

                # Check lock file is committed into Git
                if repo is not None:
                    package_lock_file = os.path.join(
                        os.path.split(package_json)[0], "package-lock.json"
                    )
                    yarn_lock_file = os.path.join(
                        os.path.split(package_json)[0], "yarn.lock"
                    )
                    pnpm_lock_file = os.path.join(
                        os.path.split(package_json)[0], "pnpm-lock.yaml"
                    )
                    is_file_committed = any(
                        f.type == "blob"
                        and os.path.join(repo.working_dir, f.path)
                        in [package_lock_file, yarn_lock_file, pnpm_lock_file]
                        for f in repo.tree("HEAD").list_traverse()
                    )
                    yield CheckResult(
                        "JS004",
                        Result.PASSED if is_file_committed else Result.FAILED,
                        package_json,
                    )
                else:
                    yield CheckResult(
                        "JS004",
                        Result.PRE_REQUISITE_CHECK_FAILED,
                        package_json,
                    )
        else:
            yield CheckResult("JS002", Result.NOT_APPLICABLE)
            yield CheckResult("JS004", Result.NOT_APPLICABLE)

    def checks(self):
        return [
            Check(
                "JS002",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team"],
                "Javascript projects should have a linter configured",
                """Javascript projects should have a comprehensive linter configured such as ESLint in order to ensure a consistent code style is used across all files and by all contributors.

Having a consistent style helps ensure readability and ease of understanding for any outsider looking into the project's code. Linters can also improve the stability of the code by catching mistakes before the code is published.""",
            ),
            Check(
                "JS004",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Javascript projects should have their lock files committed into Git",
                """When using a dependency manager for Javascript such as npm, the lock files should be comitted into Git. This ensures that all dependencies of packages are installed at the same version no matter when and on what machine the project is installed.""",
            ),
        ]



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import japr.util
import platform


class ShellCheckProvider(CheckProvider):
    def name(self):
        return "Shell"

    def test(self, directory):
        executable_files = list(japr.util.find_executable_files(directory))
        shebang_files = list(japr.util.find_files_with_shebang(directory))

        # Windows doesn't have a concept of executable files so skip these checks
        if platform.system() == "Windows" and len(executable_files) > 0:
            for executable_file in executable_files:
                yield CheckResult(
                    "SH001", Result.PRE_REQUISITE_CHECK_FAILED, executable_file
                )
        else:
            if len(executable_files) > 0:
                for executable_file in executable_files:
                    if executable_file not in shebang_files:
                        yield CheckResult(
                            "SH001",
                            Result.FAILED,
                            executable_file,
                        )
                    else:
                        yield CheckResult("SH001", Result.PASSED, executable_file)
            else:
                yield CheckResult("SH001", Result.NOT_APPLICABLE)

        if platform.system() == "Windows" and len(shebang_files) > 0:
            for shebang_file in shebang_files:
                yield CheckResult(
                    "SH002", Result.PRE_REQUISITE_CHECK_FAILED, shebang_file
                )
        else:
            if len(shebang_files) > 0:
                for shebang_file in shebang_files:
                    if shebang_file not in executable_files:
                        yield CheckResult(
                            "SH002",
                            Result.FAILED,
                            shebang_file,
                        )
                    else:
                        yield CheckResult("SH002", Result.PASSED, shebang_file)
            else:
                yield CheckResult("SH002", Result.NOT_APPLICABLE)

    def checks(self):
        return [
            Check(
                "SH001",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Shell scripts marked as executable should contain a shebang",
                """Shell scripts should start with a shebang (e.g., `#!/bin/bash` or `#!/usr/bin/env bash`) to specify the interpreter that should be used to execute the script. This ensures that the script runs correctly regardless of the user's environment.

                This check cannot be run on Windows""",
            ),
            Check(
                "SH002",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Shell scripts starting with a shebang should be marked as executable",
                """Shell scripts starting with a shebang (e.g., `#!/bin/bash` or `#!/usr/bin/env bash`) are designed to be executable from the command line. On platforms like Linux and OSX these scripts need to be marked as executable.

                 To set the executable flag on the file, use: chmod +x <script_name>

                 This check cannot be run on Windows""",
            ),
        ]



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import japr.util
from git import InvalidGitRepositoryError
from git.repo import Repo
import toml
import os


# Sourced from https://github.com/vintasoftware/python-linters-and-code-analysis
_linters = [
    "coala-bears",
    "yala",
    "prospector",
    "pylama",
    "ciocheck",
    "wemake-python-styleguide",
    "flake8",
    "black",
]


def _extract_dependencies_from_pyproject(path):
    with open(path, "r") as f:
        data = toml.load(f)

        try:
            dependencies = data["tool"]["poetry"]["dependencies"].keys()
        except KeyError:
            dependencies = []

        try:
            dev_dependencies = data["tool"]["poetry"]["group"]["dev"][
                "dependencies"
            ].keys()
        except KeyError:
            dev_dependencies = []

        return set(dependencies).union(set(dev_dependencies))


def _extract_dependencies_from_pipfile(path):
    with open(path, "r") as f:
        data = toml.load(f)

        try:
            dependencies = data["packages"].keys()
        except KeyError:
            dependencies = []

        try:
            dev_dependencies = data["dev-packages"].keys()
        except KeyError:
            dev_dependencies = []

        return set(dependencies).union(set(dev_dependencies))


class PythonCheckProvider(CheckProvider):
    def name(self):
        return "Python"

    def test(self, directory):
        requirements_txts = list(
            japr.util.find_files_with_name(directory, "requirements.txt")
        )
        pyproject_tomls = list(
            japr.util.find_files_with_name(directory, "pyproject.toml")
        )
        pipfiles = list(japr.util.find_files_with_name(directory, "Pipfile"))
        setup_pys = list(japr.util.find_files_with_name(directory, "setup.py"))
        setup_cfgs = list(japr.util.find_files_with_name(directory, "setup.cfg"))

        try:
            repo = Repo(directory, search_parent_directories=True)
        except InvalidGitRepositoryError:
            repo = None  # Deal with later when we know what checks we're doing

        if len(requirements_txts) != 0:
            for requirements_txt in requirements_txts:
                yield CheckResult("PY001", Result.FAILED, requirements_txt)
        elif len(pyproject_tomls) != 0 or len(pipfiles) != 0:
            yield CheckResult("PY001", Result.PASSED)
        else:
            yield CheckResult("PY001", Result.NOT_APPLICABLE)

        if len(pyproject_tomls) != 0 or len(pipfiles) != 0:
            for pyproject_toml in pyproject_tomls:
                dependencies = _extract_dependencies_from_pyproject(
                    os.path.join(directory, pyproject_toml)
                )
                yield CheckResult(
                    "PY002",
                    (
                        Result.PASSED
                        if len(set(_linters).intersection(dependencies))
                        else Result.FAILED
                    ),
                    pyproject_toml,
                )

                # Check lock file is committed into Git
                if repo is not None:
                    lock_file = os.path.join(
                        os.path.split(pyproject_toml)[0], "poetry.lock"
                    )
                    is_file_committed = any(
                        f.type == "blob"
                        and os.path.join(repo.working_dir, f.path) == lock_file
                        for f in repo.tree("HEAD").list_traverse()
                    )
                    yield CheckResult(
                        "PY004",
                        Result.PASSED if is_file_committed else Result.FAILED,
                        pyproject_toml,
                    )
                else:
                    yield CheckResult(
                        "PY004",
                        Result.PRE_REQUISITE_CHECK_FAILED,
                        pyproject_toml,
                    )

            for pipfile in pipfiles:
                dependencies = _extract_dependencies_from_pipfile(
                    os.path.join(directory, pipfile)
                )
                yield CheckResult(
                    "PY002",
                    (
                        Result.PASSED
                        if len(set(_linters).intersection(dependencies))
                        else Result.FAILED
                    ),
                    pipfile,
                )

                # Check lock file is committed into Git
                if repo is not None:
                    lock_file = pipfile + ".lock"
                    is_file_committed = any(
                        f.type == "blob" and f.path == lock_file
                        for f in repo.tree("HEAD").list_traverse()
                    )
                    yield CheckResult(
                        "PY004",
                        Result.PASSED if is_file_committed else Result.FAILED,
                        pipfile,
                    )
                else:
                    yield CheckResult(
                        "PY004",
                        Result.PRE_REQUISITE_CHECK_FAILED,
                        pipfile,
                    )
        else:
            yield CheckResult("PY002", Result.NOT_APPLICABLE)
            yield CheckResult("PY004", Result.NOT_APPLICABLE)

        if len(setup_pys) != 0 or len(setup_cfgs) != 0:
            for setup_py in setup_pys:
                yield CheckResult("PY003", Result.FAILED, setup_py)
            for setup_cfg in setup_cfgs:
                yield CheckResult("PY003", Result.FAILED, setup_cfg)
        elif len(pyproject_tomls) != 0 or len(pipfiles) != 0:
            yield CheckResult("PY003", Result.PASSED)
        else:
            yield CheckResult("PY003", Result.NOT_APPLICABLE)

    def checks(self):
        return [
            Check(
                "PY001",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Python projects should prefer a build system to a requirements.txt",
                """Python is moving towards using more intelligent build systems like Poetry or pipenv to manage dependencies. Consider switching from a requirements.txt file to one of these tools.""",
            ),
            Check(
                "PY002",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team"],
                "Python projects should have a linter configured",
                """Python projects should have a comprehensive linter configured such as Pylama in order to ensure a consistent code style is used across all files and by all contributors.

Having a consistent style helps ensure readability and ease of understanding for any outsider looking into the project's code. Linters can also improve the stability of the code by catching mistakes before the code is published.""",
            ),
            Check(
                "PY003",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Python projects should prefer a build system to setup.py/setup.cfg",
                """Python is moving towards using more intelligent build systems like Poetry or pipenv to manage dependencies. Consider switching from a setup.py or setup.cfg file to one of these tools.""",
            ),
            Check(
                "PY004",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                (
                    "Python projects using a dependency manager should have their lock"
                    " files committed into Git"
                ),
                """When using a dependency manager for Python such as Poetry, the lock files should be comitted into Git. This ensures that all dependencies of packages are installed at the same version no matter when and on what machine the project is installed.""",
            ),
        ]



from abc import ABC, abstractmethod
from enum import Enum


class CheckProvider(ABC):
    @abstractmethod
    def name(self):
        pass

    @abstractmethod
    def test(self, directory):
        pass

    @abstractmethod
    def checks(self):
        pass


class Check:
    def __init__(self, id, severity, project_types, reason, advice):
        self.id = id
        self.severity = severity
        self.project_types = project_types
        self.reason = reason
        self.advice = advice


class CheckResult:
    def __init__(self, id, result, file_path=None, fix=None):
        self.id = id
        self.result = result
        self.file_path = file_path
        self.fix = fix
        self.is_fixed = None


class CheckFix(ABC):
    @abstractmethod
    def fix(self, directory, file_path):
        pass

    @property
    @abstractmethod
    def success_message(self):
        pass

    @property
    @abstractmethod
    def failure_message(self):
        pass


class Severity(Enum):
    LOW = 1
    MEDIUM = 2
    HIGH = 4


class Result(Enum):
    PASSED = 0
    FAILED = 1
    PRE_REQUISITE_CHECK_FAILED = 2
    NOT_APPLICABLE = 3



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import os


CI_PATHS = [
    ".gitlab-ci.yml",
    ".travis.yml",
    "appveyor.yml",
    ".appveyor.yml",
    "circle.yml",
    ".circleci/config.yml",
    "Jenkinsfile",
    ".drone.yml",
    "azure-pipelines.yml",
    "bitbucket-pipelines.yml",
    ".buildkite/pipeline.yml",
]


class CiCheckProvider(CheckProvider):
    def name(self):
        return "CI"

    def test(self, directory):
        ci_path = None

        for path in CI_PATHS:
            if os.path.isfile(os.path.join(directory, path)):
                ci_path = os.path.join(directory, path)
        if os.path.isdir(os.path.join(directory, ".github/workflows/")):
            ci_path = os.path.join(directory, ".github/workflows/")

        yield CheckResult(
            "CI001", Result.PASSED if ci_path is not None else Result.FAILED
        )

    def checks(self):
        return [
            Check(
                "CI001",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                (
                    "Projects should define a CI/CD pipeline to ensure code builds and"
                    " works correctly"
                ),
                """Consider creating a CI/CD pipeine for this project using a tool like GitHub Actions. A typical CI/CD pipeline should:
- Lint the code
- Build the code
- Run all tests
- Deploy any artifacts like NuGet packages/PyPI packages

If you are using GitHub and would like to get started with it, you can learn how to use it here: https://docs.github.com/en/actions/quickstart""",
            )
        ]



from japr.check import Check, CheckProvider, CheckFix, CheckResult, Result, Severity
import japr.template_util
import os


class AddContributorFix(CheckFix):
    def fix(self, directory, _):
        with open(os.path.join(directory, "CONTRIBUTING.md"), "w") as f:
            f.write(japr.template_util.template("CONTRIBUTING.md", directory))
        return True

    @property
    def success_message(self):
        return (
            "Created a CONTRIBUTING.md file in the root directory from a template. You"
            " should add your own content to it."
        )

    @property
    def failure_message(self):
        return (
            "Tried to create a CONTRIBUTING.md file in the root directory but was"
            " unable to."
        )


class ContributingCheckProvider(CheckProvider):
    def name(self):
        return "Contributing"

    def test(self, directory):
        yield CheckResult(
            "CT001",
            (
                Result.PASSED
                if any(
                    [
                        os.path.isfile(os.path.join(directory, "CONTRIBUTING.md")),
                        os.path.isfile(os.path.join(directory, "CONTRIBUTING")),
                        os.path.isfile(os.path.join(directory, "CONTRIBUTING.txt")),
                        os.path.isfile(os.path.join(directory, "CONTRIBUTING.rst")),
                    ]
                )
                else Result.FAILED
            ),
            fix=AddContributorFix(),
        )

    def checks(self):
        return [
            Check(
                "CT001",
                Severity.MEDIUM,
                ["open-source"],
                (
                    "Projects should have a CONTRIBUTING.md file describing how to"
                    " contribute to the project"
                ),
                """Create a CONTRIBUTING.md file in the root of the project and add content to describe to other users how they can contribute to the project in the most helpful way

A good contributing guide should:
- Explain how people can best ask questions
- Explain how people can best report bugs or suggest enhancements
- Explain how people can donate
- Explain what the process is for making a pull request (e.g. does an issue need to exist before a pull request can be opened?)
- Explain the standards expected of any pull requests
- Explain any subtleties to contributing to the documentation
- Explain how people can join your team or help contribute in a grander way
""",
            )
        ]



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import japr.util
import os
import xml.etree.ElementTree as ET


_linters = [
    "StyleCop.Analyzers",
    "SonarAnalyzer.CSharp",
    "Microsoft.CodeAnalysis.NetAnalyzers",
    "Roslynator.Analyzers",
    "Roslynator.CodeAnalysis.Analyzers",
    "Roslynator.Formatting.Analyzers",
]


def _extract_dependencies_from_csproj(file):
    data = ET.parse(file)

    try:
        dependencies = [
            reference.get("Include")
            for reference in data.findall("./ItemGroup/PackageReference")
        ]
    except KeyError:
        print("ERROR")
        dependencies = []

    return set(dependencies)


def _has_enabled_net_analyzers_in_csproj(file):
    data = ET.parse(file)

    try:
        # TODO this defaults to true on net5.0 and above
        # TODO consider the EnforceCodeStyleInBuild property too
        properties = data.findall("./PropertyGroup/EnableNETAnalyzers")
        return len(properties) > 0 and all(
            property.text == "true" for property in properties
        )
    except KeyError:
        return False


def _has_enabled_nullable_in_csproj(file):
    data = ET.parse(file)

    try:
        properties = data.findall("./PropertyGroup/Nullable")
        return len(properties) > 0 and all(
            property.text == "enable" for property in properties
        )
    except KeyError:
        return False


def _has_enabled_treat_warnings_as_errors(file):
    data = ET.parse(file)

    try:
        properties = data.findall("./PropertyGroup/TreatWarningsAsErrors")
        return len(properties) > 0 and all(
            property.text == "true" for property in properties
        )
    except KeyError:
        return False


class CSharpCheckProvider(CheckProvider):
    def name(self):
        return "C#"

    def test(self, directory):
        cs_projects = list(japr.util.find_files_with_extension(directory, "csproj"))

        if len(cs_projects) != 0:
            for cs_project in cs_projects:
                dependencies = _extract_dependencies_from_csproj(
                    os.path.join(directory, cs_project)
                )
                has_enabled_net_analyzers = _has_enabled_net_analyzers_in_csproj(
                    os.path.join(directory, cs_project)
                )
                yield CheckResult(
                    "CS002",
                    (
                        Result.PASSED
                        if len(set(_linters).intersection(dependencies))
                        or has_enabled_net_analyzers
                        else Result.FAILED
                    ),
                    cs_project,
                )
                has_enabled_nullable = _has_enabled_nullable_in_csproj(
                    os.path.join(directory, cs_project)
                )
                yield CheckResult(
                    "CS003",
                    Result.PASSED if has_enabled_nullable else Result.FAILED,
                    cs_project,
                )
                has_enabled_treat_warnings_as_errors = (
                    _has_enabled_treat_warnings_as_errors(
                        os.path.join(directory, cs_project)
                    )
                )
                yield CheckResult(
                    "CS004",
                    (
                        Result.PASSED
                        if has_enabled_treat_warnings_as_errors
                        else Result.FAILED
                    ),
                    cs_project,
                )
        else:
            yield CheckResult("CS002", Result.NOT_APPLICABLE)
            yield CheckResult("CS003", Result.NOT_APPLICABLE)
            yield CheckResult("CS004", Result.NOT_APPLICABLE)

    def checks(self):
        return [
            Check(
                "CS002",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team"],
                "C# projects should have a linter configured",
                """C# projects should have a comprehensive linter configured such as StyleCop in order to ensure a consistent code style is used across all files and by all contributors.

Having a consistent style helps ensure readability and ease of understanding for any outsider looking into the project's code. Linters can also improve the stability of the code by catching mistakes before the code is published.""",
            ),
            Check(
                "CS003",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team"],
                "C# projects should have nullable reference types enabled",
                """C# projects should have nullable reference types enabled to avoid hard to detect null reference exceptions from appearing during runtime.

Nullable reference types were introduced in C#8 and can be enabled by adding the following into your csproj file:
    <PropertyGroup>
      <Nullable>enable</Nullable>
    </PropertyGroup>

See https://learn.microsoft.com/en-us/dotnet/csharp/nullable-references""",
            ),
            Check(
                "CS004",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team"],
                "C# projects should have treat warnings as errors enabled",
                """C# projects should have treat warnings as errors enabled to avoid warnings being ignored. This will cause warnings to fail the build and stop compilation

Warnings can be set to fail the build using:
    <PropertyGroup>
      <TreatWarningsAsErrors>enable</TreatWarningsAsErrors>
    </PropertyGroup>
""",
            ),
        ]



from japr.check import Check, CheckProvider, CheckFix, CheckResult, Result, Severity
from git import InvalidGitRepositoryError
from git.repo import Repo
import japr.template_util
import japr.util
import os
import yaml


class AddIssueTemplateFix(CheckFix):
    def fix(self, directory, _):
        os.makedirs(os.path.join(directory, ".github/ISSUE_TEMPLATE"), exist_ok=True)
        with open(os.path.join(directory, ".github/ISSUE_TEMPLATE/bug.md"), "w") as f:
            f.write(
                japr.template_util.template("bug_report_issue_template.md", directory)
            )
        with open(
            os.path.join(directory, ".github/ISSUE_TEMPLATE/feature_request.md"), "w"
        ) as f:
            f.write(
                japr.template_util.template(
                    "feature_request_issue_template.md", directory
                )
            )
        return True

    @property
    def success_message(self):
        return (
            "Created issue templates at .github/ISSUE_TEMPLATE/bug_report.md and"
            " .github/ISSUE_TEMPLATE/feature_request.md from a template. You should add"
            " your own content to it."
        )

    @property
    def failure_message(self):
        return (
            "Tried to create an issue template at .github/ISSUE_TEMPLATE/bug_report.md"
            " and .github/ISSUE_TEMPLATE/feature_request.md but was unable to."
        )


class AddPullRequestTemplateFix(CheckFix):
    def fix(self, directory, _):
        os.makedirs(os.path.join(directory, ".github"), exist_ok=True)
        with open(
            os.path.join(directory, ".github/pull_request_template.md"), "w"
        ) as f:
            f.write(japr.template_util.template("pull_request_template.md", directory))
        return True

    @property
    def success_message(self):
        return (
            "Created an issue template at .github/pull_request_template.md from a"
            " template. You should add your own content to it."
        )

    @property
    def failure_message(self):
        return (
            "Tried to create an issue template at .github/pull_request_template.md but"
            " was unable to."
        )


class GitHubCheckProvider(CheckProvider):
    def name(self):
        return "GitHub"

    def test(self, directory):
        try:
            repo = Repo(directory, search_parent_directories=True)
            github_is_origin = (
                "origin" in repo.remotes and "github" in repo.remote("origin").url
            )
        except InvalidGitRepositoryError:
            github_is_origin = False

        if not github_is_origin:
            yield CheckResult("GH001", Result.NOT_APPLICABLE)
            yield CheckResult("GH002", Result.NOT_APPLICABLE)
            return

        # https://docs.github.com/en/communities/using-templates-to-encourage-useful-issues-and-pull-requests/about-issue-and-pull-request-templates
        has_issue_template = any(
            [
                os.path.isfile(os.path.join(directory, "issue_template")),
                os.path.isfile(os.path.join(directory, "issue_template.md")),
                os.path.isfile(os.path.join(directory, "issue_template.yml")),
                os.path.isfile(os.path.join(directory, "docs/issue_template")),
                os.path.isfile(os.path.join(directory, "docs/issue_template.md")),
                os.path.isfile(os.path.join(directory, "docs/issue_template.yml")),
                os.path.isfile(os.path.join(directory, ".github/issue_template")),
                os.path.isfile(os.path.join(directory, ".github/issue_template.md")),
                os.path.isfile(os.path.join(directory, ".github/issue_template.yml")),
                os.path.isdir(os.path.join(directory, "issue_template")),
                os.path.isdir(os.path.join(directory, "docs/issue_template")),
                os.path.isdir(os.path.join(directory, ".github/issue_template")),
            ]
        )
        # https://docs.github.com/en/communities/using-templates-to-encourage-useful-issues-and-pull-requests/creating-a-pull-request-template-for-your-repository
        has_pull_request_template = any(
            [
                os.path.isfile(os.path.join(directory, "pull_request_template")),
                os.path.isfile(os.path.join(directory, "pull_request_template.md")),
                os.path.isfile(os.path.join(directory, "docs/pull_request_template")),
                os.path.isfile(
                    os.path.join(directory, "docs/pull_request_template.md")
                ),
                os.path.isfile(
                    os.path.join(directory, ".github/pull_request_template")
                ),
                os.path.isfile(
                    os.path.join(directory, ".github/pull_request_template.md")
                ),
                os.path.isdir(os.path.join(directory, "pull_request_template")),
                os.path.isdir(os.path.join(directory, "docs/pull_request_template")),
                os.path.isdir(os.path.join(directory, ".github/pull_request_template")),
            ]
        )

        yield CheckResult(
            "GH001",
            Result.PASSED if has_issue_template else Result.FAILED,
            fix=AddIssueTemplateFix(),
        )
        yield CheckResult(
            "GH002",
            Result.PASSED if has_pull_request_template else Result.FAILED,
            fix=AddPullRequestTemplateFix(),
        )

        has_workflow_job = False
        if os.path.isdir(os.path.join(directory, ".github", "workflows")):
            workflows = japr.util.find_files_with_extensions(
                os.path.join(directory, ".github", "workflows"), ["yaml", "yml"]
            )

            for workflow in workflows:
                with open(workflow, "r") as f:
                    workflow_yaml = yaml.load(f, yaml.SafeLoader)

                    jobs = workflow_yaml.get("jobs", [])
                    if len(jobs) > 0:
                        has_workflow_job = True

                    has_job_timeouts = all(
                        "timeout-minutes" in jobs[job] for job in jobs
                    )

                    yield CheckResult(
                        "GH003",
                        Result.PASSED if has_job_timeouts else Result.FAILED,
                        file_path=workflow,
                    )

        if not has_workflow_job:
            yield CheckResult("GH003", Result.NOT_APPLICABLE)

    def checks(self):
        return [
            Check(
                "GH001",
                Severity.LOW,
                ["open-source", "inner-source"],
                "GitHub projects should have an issue template",
                """To help users create issues that are useful for you an issue template is recommended.

Create a .github/issue_template.md file and fill it with a template for users to use when filing issues.
See https://docs.github.com/en/communities/using-templates-to-encourage-useful-issues-and-pull-requests/about-issue-and-pull-request-templates""",
            ),
            Check(
                "GH002",
                Severity.LOW,
                ["open-source", "inner-source"],
                "GitHub projects should have a pull request template",
                """To help users create pull requests that are useful for you a pull request template is recommended.

Create a .github/pull_request_template.md file and fill it with a template for users to use when filing pull requests
See https://docs.github.com/en/communities/using-templates-to-encourage-useful-issues-and-pull-requests/creating-a-pull-request-template-for-your-repository""",
            ),
            Check(
                "GH003",
                Severity.HIGH,
                ["open-source", "inner-source", "team", "personal"],
                "GitHub Actions workflows should have timeouts set",
                """Workflows for GitHub Actions must have timeouts set to avoid hefty charges getting incurred by stuck jobs. By default, the timeout is 6h which is lmost always unecessary

                Add the timeout-minutes property to your job
                See https://docs.github.com/en/actions/writing-workflows/workflow-syntax-for-github-actions#jobsjob_idtimeout-minutes""",
            ),
        ]



from japr.check import Check, CheckProvider, CheckResult, Result, Severity
import japr.util
from git import InvalidGitRepositoryError
from git.repo import Repo
import os


def _is_subdir(path, directory):
    path = os.path.realpath(path)
    directory = os.path.realpath(directory)

    relative = os.path.relpath(path, directory)

    if relative.startswith(os.pardir):
        return False
    else:
        return True


class TerraformCheckProvider(CheckProvider):
    def name(self):
        return "Terraform"

    def test(self, directory):
        terraform_locks = list(
            japr.util.find_files_with_name(directory, ".terraform.lock.hcl")
        )

        try:
            repo = Repo(directory, search_parent_directories=True)
        except InvalidGitRepositoryError:
            repo = None  # Deal with later when we know what checks we're doing

        if len(terraform_locks) != 0:
            for terraform_lock in terraform_locks:
                # Check lock file is committed into Git
                if repo is not None:
                    is_file_committed = any(
                        f.type == "blob"
                        and os.path.join(repo.working_dir, f.path) == terraform_lock
                        for f in repo.tree("HEAD").list_traverse()
                    )
                    yield CheckResult(
                        "TF004",
                        Result.PASSED if is_file_committed else Result.FAILED,
                        terraform_lock,
                    )

                    dot_terraform_dir = os.path.join(
                        os.path.dirname(terraform_lock), ".terraform"
                    )
                    is_dot_terraform_dir_committed = any(
                        f.type == "tree"
                        and os.path.join(repo.working_dir, f.path) == dot_terraform_dir
                        for f in repo.tree("HEAD").list_traverse()
                    )
                    yield CheckResult(
                        "TF005",
                        (
                            Result.PASSED
                            if not is_dot_terraform_dir_committed
                            else Result.FAILED
                        ),
                        dot_terraform_dir,
                    )
                else:
                    yield CheckResult(
                        "TF004",
                        Result.PRE_REQUISITE_CHECK_FAILED,
                        terraform_lock,
                    )
                    yield CheckResult(
                        "TF005",
                        Result.PRE_REQUISITE_CHECK_FAILED,
                        terraform_lock,
                    )
        else:
            yield CheckResult("TF004", Result.NOT_APPLICABLE)
            yield CheckResult("TF005", Result.NOT_APPLICABLE)

        terraform_state_files = list(
            japr.util.find_files_with_extension(directory, ".tfstate")
        )
        if len(terraform_state_files) != 0:
            for terraform_state_file in terraform_state_files:
                # Check state file is not committed into Git
                if repo is not None:
                    is_file_committed = any(
                        f.type == "blob"
                        and os.path.join(repo.working_dir, f.path)
                        == terraform_state_file
                        for f in repo.tree("HEAD").list_traverse()
                    )
                    yield CheckResult(
                        "TF006",
                        Result.PASSED if not is_file_committed else Result.FAILED,
                        terraform_state_file,
                    )
                else:
                    yield CheckResult(
                        "TF006",
                        Result.PRE_REQUISITE_CHECK_FAILED,
                        terraform_state_file,
                    )
        else:
            yield CheckResult("TF006", Result.NOT_APPLICABLE)

        terraform_files = list(japr.util.find_files_with_extension(directory, ".tf"))
        terraform_dirs = set(
            [os.path.dirname(terraform_file) for terraform_file in terraform_files]
        )
        if len(terraform_dirs) != 0:
            for terraform_dir in terraform_dirs:
                # Check required files are present
                has_main_file = (
                    os.path.join(terraform_dir, "main.tf") in terraform_files
                )
                yield CheckResult(
                    "TF007",
                    Result.PASSED if has_main_file else Result.FAILED,
                    terraform_dir,
                )
                has_outputs_file = (
                    os.path.join(terraform_dir, "outputs.tf") in terraform_files
                )
                yield CheckResult(
                    "TF008",
                    Result.PASSED if has_outputs_file else Result.FAILED,
                    terraform_dir,
                )
                has_variables_file = (
                    os.path.join(terraform_dir, "variables.tf") in terraform_files
                )
                yield CheckResult(
                    "TF009",
                    Result.PASSED if has_variables_file else Result.FAILED,
                    terraform_dir,
                )

                # If we're a subdir of any other directory there should be a /modules/ directory between us
                # Find closest parent from the other dirs
                closest_parent = None
                for other_dir in terraform_dirs:
                    if terraform_dir != other_dir and _is_subdir(
                        terraform_dir, other_dir
                    ):
                        if closest_parent is None or _is_subdir(
                            closest_parent, other_dir
                        ):
                            closest_parent = other_dir

                if closest_parent is not None:
                    is_in_modules_dir = os.path.join(
                        closest_parent, "modules"
                    ) == os.path.dirname(terraform_dir)

                    yield CheckResult(
                        "TF010",
                        Result.PASSED if is_in_modules_dir else Result.FAILED,
                        terraform_dir,
                    )
                else:
                    yield CheckResult("TF010", Result.NOT_APPLICABLE, terraform_dir)

        else:
            yield CheckResult("TF007", Result.NOT_APPLICABLE)
            yield CheckResult("TF008", Result.NOT_APPLICABLE)
            yield CheckResult("TF009", Result.NOT_APPLICABLE)
            yield CheckResult("TF010", Result.NOT_APPLICABLE)

    def checks(self):
        return [
            Check(
                "TF004",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Terraform projects should have their Terraform lock files committed into Git",
                """When using Terraform, the lock files should be comitted into Git. This ensures that all dependencies of packages are installed at the same version no matter when and on what machine the project is installed.""",
            ),
            Check(
                "TF005",
                Severity.MEDIUM,
                ["open-source", "inner-source", "team", "personal"],
                "Terraform projects should not have their .terraform directory committed into Git",
                """The .terraform directory contains binaries, thrid party modules and other things that are generated during a terraform init. This folder should not be committed into git.""",
            ),
            Check(
                "TF006",
                Severity.HIGH,
                ["open-source", "inner-source", "team", "personal"],
                "Terraform state files should not be committed into git",
                """Terraform state files can contain secrets and are stored unencrypted. These secrets can be easily leaked when stored in git. Additionally, terraform state in git does not provide a single source of truth nor state locking and so can cause major issues when used in teams.

Instead of storing state in git, use another terraform backend

See https://developer.hashicorp.com/terraform/language/settings/backends/configuration""",
            ),
            Check(
                "TF007",
                Severity.LOW,
                ["open-source", "inner-source", "team", "personal"],
                "Terraform modules should contain a main.tf file",
                """When creating terraform modules (or using terraform in general) each module should contain a minimum of a main.tf, outputs.tf and a variables.tf file, even if these are empty.

See https://developer.hashicorp.com/terraform/language/modules/develop/structure""",
            ),
            Check(
                "TF008",
                Severity.LOW,
                ["open-source", "inner-source", "team", "personal"],
                "Terraform modules should contain an outputs.tf file",
                """When creating terraform modules (or using terraform in general) each module should contain a minimum of a main.tf, outputs.tf and a variables.tf file, even if these are empty.

See https://developer.hashicorp.com/terraform/language/modules/develop/structure""",
            ),
            Check(
                "TF009",
                Severity.LOW,
                ["open-source", "inner-source", "team", "personal"],
                "Terraform modules should contain a variables.tf file",
                """When creating terraform modules (or using terraform in general) each module should contain a minimum of a main.tf, outputs.tf and a variables.tf file, even if these are empty.

See https://developer.hashicorp.com/terraform/language/modules/develop/structure""",
            ),
            Check(
                "TF010",
                Severity.LOW,
                ["open-source", "inner-source", "team", "personal"],
                "Terraform submodules should be contained in a 'modules' directory",
                """When creating submodules in a terraform module or project the submodules should be contained in a 'modules' directory. For example:
root/
|- modules/
|  '- my-submodule/
|     |-main.tf
|     |-outputs.tf
|     '-variables.tf
|-main.tf
|-outputs.tf
'-variables.tf

See https://developer.hashicorp.com/terraform/language/modules/develop/structure""",
            ),
        ]



import os
import stat

# List of directories to immediately stop searching when looking for files
# This is designed to be a short and non-comprehensive list mostly in the interest of search speed
SKIP_DIRECTORIES = [
    # You know why this is skipped...
    "node_modules",
    # Stores git history as lots and lots of files we don't want to parse through
    ".git",
    # Stores installed terraform modules and other autogenerated stuff
    ".terraform",
]


def _walk(base_path):
    with os.scandir(base_path) as scan:
        for obj in scan:
            if obj.is_dir() and obj.name not in SKIP_DIRECTORIES:
                yield from _walk(obj.path)
            elif obj.is_file():
                yield (obj.name, obj.path)


def find_files_with_extension(base_path, extension):
    yield from (
        obj_path
        for obj_name, obj_path in _walk(base_path)
        if obj_name.endswith(extension)
    )


def find_files_with_extensions(base_path, extensions):
    for extension in extensions:
        yield from find_files_with_extension(base_path, extension)


def find_files_with_shebang(base_path):
    def has_shebang(path):
        with open(path, "r") as f:
            first_line = f.readline().strip()
            return first_line.startswith("#!")

    yield from (obj_path for _, obj_path in _walk(base_path) if has_shebang(obj_path))


def find_executable_files(base_path):
    yield from (
        obj_path
        for _, obj_path in _walk(base_path)
        if "x" in stat.filemode(os.stat(obj_path).st_mode)
    )


def find_files_with_name(base_path, name):
    yield from (obj_path for obj_name, obj_path in _walk(base_path) if obj_name == name)



import argparse
from japr.check_providers import check_providers
from japr.check import Result, Severity
import json
import os
import sys
import time
import yaml


PROJECT_TYPES = ["open-source", "inner-source", "team", "personal"]


class Japr:
    def __init__(self, is_summary=False, is_profile=False, fix=False, is_json=False):
        self.is_summary = is_summary
        self.is_profile = is_profile
        self.fix = fix
        self.is_json = is_json

        if self.fix:
            self.is_summary = True

    def _print_result(self, result, check, profile_time, suppressed_checks):
        if (
            result.result == Result.FAILED and result.id not in suppressed_checks
        ) or self.is_profile:
            # Build up components then display for code clarity
            if result.result == Result.FAILED:
                emoji_block = "\N{CROSS MARK}"
            elif result.result == Result.PASSED:
                emoji_block = "\N{WHITE HEAVY CHECK MARK}"
            elif result.result == Result.PRE_REQUISITE_CHECK_FAILED:
                emoji_block = "\N{WHITE QUESTION MARK ORNAMENT}"
            else:
                emoji_block = "\N{HEAVY MINUS SIGN}"

            if check.severity == Severity.HIGH:
                severity_color = "\033[1;31m"
            elif check.severity == Severity.MEDIUM:
                severity_color = "\033[1;33m"
            else:
                severity_color = "\033[37m"

            profile_block = (str(round(profile_time * 1000)) + "ms").ljust(8)

            if result.file_path is not None:
                file_block = f"[{result.file_path}] "
            else:
                file_block = ""

            if result.fix is not None:
                fix_block = f" - A fix is available \N{WRENCH}"
            else:
                fix_block = ""

            if self.is_summary:
                print(
                    f"{severity_color}{check.severity.name.ljust(6)}\033[0;0m -"
                    f" \033[1m{check.id}\033[0;0m {file_block}{check.reason}"
                    f"{fix_block}"
                )
            elif self.is_profile:
                print(
                    f"{emoji_block} {severity_color}{check.severity.name.ljust(6)}\033[0;0m"
                    f" - {profile_block} - \033[1m{check.id}\033[0;0m"
                    f" {file_block}{check.reason}{fix_block}"
                )
            else:
                print(
                    f"{severity_color}{check.severity.name.ljust(6)}\033[0;0m -"
                    f" \033[1m{check.id}\033[0;0m {file_block}{check.reason}"
                    f"{fix_block}"
                )
                print()
                print(check.advice)
                print()
                print("-" * 10)

    def _print_fix_result(self, result):
        if result.is_fixed:
            print(f"\N{WHITE HEAVY CHECK MARK} {result.fix.success_message}")
        else:
            print(f"\N{CROSS MARK} {result.fix.failure_message}")

    def _print_json(
        self,
        issues,
        score,
        passed,
        failed,
        cannot_run,
        suppressed,
        fixed,
        failed_to_fix,
        suppressed_checks,
    ):
        out = {
            "results": [
                {
                    "id": result.id,
                    "result": result.result.name,
                    "filePath": result.file_path,
                    "isFixAvailable": result.fix is not None,
                    "severity": check.severity.name,
                    "reason": check.reason,
                    "advice": check.advice,
                    "isSuppressed": result.id in suppressed_checks,
                    "isFixed": result.is_fixed,
                }
                for (result, check, _) in issues
            ],
            "score": score,
            "passed": passed,
            "failed": failed,
            "cannotRun": cannot_run,
            "suppressed": suppressed,
            "fixed": fixed,
            "failedToFix": failed_to_fix,
        }
        print(json.dumps(out, indent=2))

    def _print_summary(
        self, score, passed, failed, cannot_run, suppressed, fixed, failed_to_fix
    ):
        print(
            "\033[1mProject score: "
            + "\N{GLOWING STAR}" * score
            + "\N{HEAVY MINUS SIGN}" * (5 - score)
            + "\033[0;0m"
        )

        print(
            f"\033[1m\033[1;32mPassed: {passed}\033[0;0m, \033[1m\033[1;31mFailed:"
            f" {failed}\033[0;0m, \033[1m\033[1;37mCannot Run Yet: {cannot_run},"
            f" Suppressed {suppressed}\033[0;0m"
        )

        if self.fix:
            print(
                f"\033[1m\033[1;32mFixed: {fixed}\033[0;0m, \033[1m\033[1;31mFailed to"
                f" Fix: {failed_to_fix}\033[0;0m"
            )

        if score == 5:
            print()
            print("\033[1mCongratulations on a fantastic score \U0001f389\033[0;0m")

    def check_directory(
        self,
        directory,
        project_type,
    ):
        directory = os.path.abspath(directory)
        if not os.path.isdir(directory):
            print(
                f"'{directory}' is not a valid directory so cannot be checked",
                file=sys.stderr,
            )
            return

        if os.path.isfile(os.path.join(directory, ".japr.yaml")):
            with open(os.path.join(directory, ".japr.yaml"), "r") as f:
                data = yaml.safe_load(f)
                try:
                    suppressed_checks = [
                        override["id"]
                        for override in data["override"]
                        if override["suppress"]
                    ]
                except KeyError:
                    suppressed_checks = []
                if project_type is None:
                    try:
                        project_type = data["projectType"]
                    except KeyError:
                        project_type = None
        else:
            suppressed_checks = []

        if project_type is None:
            print(
                (
                    "No project type specified. You can specify this with the -t flag"
                    " or add to your .japr.yaml configuration file."
                ),
                file=sys.stderr,
            )
            return

        if project_type not in PROJECT_TYPES:
            print(
                f"Invalid project type. Must be one of {', '.join(PROJECT_TYPES)}.",
                file=sys.stderr,
            )
            return

        issues = []
        for check_provider in check_providers:
            checks = {check.id: check for check in check_provider.checks()}

            results = check_provider.test(directory)

            # results is actually a generator so we can time how long it takes to make each result
            start = time.time()
            for result in results:
                end = time.time()
                try:
                    if project_type in checks[result.id].project_types:
                        check = checks[result.id]
                        profile_time = end - start
                        issues.append((result, check, profile_time))
                        if not self.is_json:
                            self._print_result(
                                result, check, profile_time, suppressed_checks
                            )
                        if (
                            self.fix
                            and result.result == Result.FAILED
                            and result.fix is not None
                        ):
                            result.is_fixed = result.fix.fix(
                                directory, result.file_path
                            )
                            if not self.is_json:
                                self._print_fix_result(result)
                except KeyError:
                    raise Exception(
                        f"Check {result.id} is not defined in the"
                        f" {check_provider.name()} check provider but a result was"
                        " returned for it. Ensure the result is returning the correct"
                        " ID and the check is defined correctly in the provider."
                    )
                start = time.time()

        sev_bad_checks = sum(
            check.severity.value
            for (result, check, _) in issues
            if result.result == Result.FAILED
            or result.result == Result.PRE_REQUISITE_CHECK_FAILED
        )
        sev_all_checks = sum(check.severity.value for (_, check, _) in issues)
        score = int(5 - sev_bad_checks / sev_all_checks * 5)

        passed = len(
            [
                result
                for (result, _, _) in issues
                if result.result == Result.PASSED and result.id not in suppressed_checks
            ]
        )
        failed = len(
            [
                result
                for (result, _, _) in issues
                if result.result == Result.FAILED and result.id not in suppressed_checks
            ]
        )
        cannot_run = len(
            [
                result
                for (result, _, _) in issues
                if result.result == Result.PRE_REQUISITE_CHECK_FAILED
                and result.id not in suppressed_checks
            ]
        )
        suppressed = len(
            [result for (result, _, _) in issues if result.id in suppressed_checks]
        )
        fixed = len([result for (result, _, _) in issues if result.is_fixed])
        failed_to_fix = len(
            [result for (result, _, _) in issues if result.is_fixed == False]
        )

        if self.is_json:
            self._print_json(
                issues,
                score,
                passed,
                failed,
                cannot_run,
                suppressed,
                fixed,
                failed_to_fix,
                suppressed_checks,
            )
        else:
            print()
            self._print_summary(
                score, passed, failed, cannot_run, suppressed, fixed, failed_to_fix
            )
            print()

        if self.fix:
            return failed_to_fix == 0
        else:
            return failed == 0


def cli(args=None):
    parser = argparse.ArgumentParser(
        prog="japr",
        description=(
            "A cross-language tool for rating the overall quality of open source,"
            " commercial and personal projects"
        ),
    )

    parser.add_argument("directory", help="the directory to scan")
    parser.add_argument(
        "-t",
        "--project-type",
        help="the type of project being scanned",
        choices=PROJECT_TYPES,
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "--profile", help="times how long each check takes to run", action="store_true"
    )
    group.add_argument(
        "-s", "--summary", help="prints results in summary form", action="store_true"
    )
    group.add_argument(
        "--fix", help="experimentally try to fix issues found", action="store_true"
    )
    group.add_argument("--json", help="output results as JSON", action="store_true")

    args = parser.parse_args(args)

    japr = Japr(
        args.summary,
        args.profile,
        args.fix,
        args.json,
    )

    if japr.check_directory(
        args.directory,
        args.project_type,
    ):
        quit(0)
    else:
        quit(1)
